#define G_LOG_DOMAIN "gpod-parser"

#include "gpod-types.h"
#include "gpod-parser.h"

#include <glib-object.h>
#include <glib.h>
#include <gio/gio.h>

/**
 * SECTION:gpod-parser
 * @title: GpodParser
 * @short_description: a RSS XML parser creating a new #GpodFeed
 * @include: gpod-parser.h
 * @see_also: #GpodFeed, #GpodChannel, #GpodItem
 *
 * The #GpodParser is an RSS XML parser to create a new #GpodFeed.
 **/

typedef struct _GpodParserPrivate
{
    gchar *temp;
} GpodParserPrivate;

G_DEFINE_TYPE_WITH_PRIVATE (GpodParser, gpod_parser, G_TYPE_OBJECT)

/*
 * gpod_parser_finalize:
 * @object: An instance of #GpodParser
 *
 * Does any last minute clean-ups when the class ends.
 */
static void
gpod_parser_finalize (GObject *object)
{
    G_OBJECT_CLASS (gpod_parser_parent_class)->finalize(object);
}

/*
 * gpod_parser_init:
 * @self: An instance of #GpodParser
 *
 * Initializes various aspects of the #GpodParser. Anything the #GpodParser
 * would need before becoming presented should be done here.
 */
static void
gpod_parser_init (GpodParser *self)
{

}

/*
 * gpod_parser_class_init:
 * @klass: An instance of #GpodParserClass
 *
 * Initializes the various virtural functions in #GpodParserClass. In
 * addtition it connects the various #GtkWidgets from the gresource file using
 * gtk_widget_class_set_template_from_resource().
 */
static void
gpod_parser_class_init (GpodParserClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->finalize = gpod_parser_finalize;
}

/**
 * gpod_parser_new:
 *
 * Creates a new #GpodParser. #GpodParser can be used to load a RSS stream from
 * either a file or a buffer and then walk the document.
 *
 * ## Usage
 * |[
 * GpodParser *parser;
 *
 * parser = gpod_parser_new ();
 * ]|
 *
 * Returns: the newly created #GpodParser. Use g_object_unref() to release all
 *          memory it allocates.
 */
GpodParser *
gpod_parser_new ()
{
    return g_object_new (GPOD_TYPE_PARSER,
                         NULL);
}

gboolean
gpod_parser_load_from_data (GpodParser  *self,
                            gchar       *data,
                            gssize       length,
                            GError     **error)
{
    g_return_val_if_fail (GPOD_IS_PARSER (self), FALSE);
    g_return_val_if_fail (data != NULL, FALSE);

    return TRUE;
}

gboolean
gpod_parser_load_from_file (GpodParser  *self,
                            gchar       *filename,
                            GError     **error)
{
    g_return_val_if_fail (GPOD_IS_PARSER (self), FALSE);
    g_return_val_if_fail (filename != NULL, FALSE);

    GpodParserPrivate *priv = gpod_parser_get_instance_private (self);
    GError *internal_error;
    gchar *data;
    gsize length;
    gboolean retval = TRUE;

internal_error = NULL;
if (!g_file_get_contents (filename, &data, &length, &internal_error))
{
g_propagate_error (error, internal_error);
return FALSE;
}

g_free (priv->filename);

priv->is_filename = TRUE;
priv->filename = g_strdup (filename);

if (!json_parser_load (parser, data, length, &internal_error))
{
g_propagate_error (error, internal_error);
retval = FALSE;
}

g_free (data);

return retval;
}

gboolean
gpod_parser_load_from_stream (GpodParser    *self,
                              GInputStream  *stream,
                              GCancellable  *cancellable,
                              GError       **error)
{
    g_return_val_if_fail (GPOD_IS_PARSER (self), FALSE);
    g_return_val_if_fail (G_IS_INPUT_STREAM (stream), FALSE);
    g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), FALSE);

    return TRUE;
}
